/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.vm.cpu.agent.internal;

import java.io.IOException;
import java.net.URI;
import java.util.Collections;
import java.util.List;
import java.util.logging.Logger;

import com.redhat.thermostat.agent.http.HttpRequestService;
import com.redhat.thermostat.common.config.experimental.ConfigurationInfoSource;
import com.redhat.thermostat.common.plugin.BackendConfigurationUtil;
import com.redhat.thermostat.common.plugin.PluginDAOBase;
import com.redhat.thermostat.common.plugin.SystemID;
import com.redhat.thermostat.common.utils.LoggingUtils;
import com.redhat.thermostat.vm.cpu.agent.model.VmCpuStat;
import com.redhat.thermostat.vm.cpu.agent.model.VmCpuStatTypeAdapter;
import org.apache.felix.scr.annotations.Activate;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.Service;

@Component
@Service(value = VmCpuStatDAO.class)
public class VmCpuStatDAOImpl extends PluginDAOBase<VmCpuStat> implements VmCpuStatDAO {

    private static final Logger logger = LoggingUtils.getLogger(VmCpuStatDAOImpl.class);
    public static final String PLUGIN_ID = "vm-cpu";

    @Reference
    private ConfigurationInfoSource configurationInfoSource;

    @Reference
    private SystemID systemID;

    @Reference
    private HttpRequestService httpRequestService;

    private final JsonHelper jsonHelper;
    private final ConfigurationCreator configurationCreator;
    private BackendConfigurationUtil pluginConfiguration;

    public VmCpuStatDAOImpl() {
        this(new JsonHelper(new VmCpuStatTypeAdapter()), new ConfigurationCreator());
    }

    VmCpuStatDAOImpl(JsonHelper jsonHelper, ConfigurationCreator configurationCreator) {
        this.jsonHelper = jsonHelper;
        this.configurationCreator = configurationCreator;
    }

    @Activate
    public void activate() throws Exception {
        pluginConfiguration = configurationCreator.create(configurationInfoSource);
    }

    @Override
    protected String toJsonString(VmCpuStat obj) throws IOException {
        return jsonHelper.toJson(Collections.singletonList(obj));
    }

    @Override
    protected HttpRequestService getHttpRequestService() {
        return httpRequestService;
    }

    @Override
    protected BackendConfigurationUtil getConfig() {
        return pluginConfiguration;
    }

    @Override
    protected URI getPostURI(URI basepath, VmCpuStat vmCpuStat) {
        return basepath.resolve("systems/" + systemID.getSystemID() + "/jvms/" + vmCpuStat.getVmId());
    }

    @Override
    protected Logger getLogger() {
        return logger;
    }

    // DS bind method.
    protected void bindSystemID(SystemID systemid) {
        this.systemID = systemid;
    }

    // DS bind method.
    protected void bindConfigurationInfoSource(ConfigurationInfoSource cfg) {
        this.configurationInfoSource = cfg;
    }

    // DS bind method.
    protected void bindHttpRequestService(HttpRequestService httpRequestService) {
        this.httpRequestService = httpRequestService;
    }

    // For testing.
    static class JsonHelper {
        private final VmCpuStatTypeAdapter typeAdapter;

        JsonHelper(VmCpuStatTypeAdapter typeAdapter) {
            this.typeAdapter = typeAdapter;
        }

        String toJson(List<VmCpuStat> infos) throws IOException {
            return typeAdapter.toJson(infos);
        }
    }

    // For testing.
    static class ConfigurationCreator {
        BackendConfigurationUtil create(ConfigurationInfoSource source) {
            return new BackendConfigurationUtil(source, PLUGIN_ID);
        }
    }
}
